#include "config.h"
#include <stdlib.h>
#include <unistd.h>
#include "connect.h"
#include "errcodes.h"
#include "fdbuf.h"
#include "hostname.h"
#include "itoa.h"
#include "mystring.h"
#include "netstring.h"

class qmqp 
{
  fdibuf in;
  fdobuf out;
public:
  qmqp(int fd);
  ~qmqp();
  int send(fdibuf& msg, unsigned long size, const mystring& env);
};

qmqp::qmqp(int fd)
  : in(fd), out(fd)
{
}

qmqp::~qmqp()
{
}

bool skip_envelope(fdibuf& msg)
{
  if(!msg.rewind())
    return false;
  mystring tmp;
  while(msg.getline(tmp))
    if(!tmp)
      break;
  return msg;
}
  
int qmqp::send(fdibuf& msg, unsigned long size, const mystring& env)
{
  if(!skip_envelope(msg))
    return -ERR_MSG_READ;
  unsigned long fullsize = strlen(itoa(size)) + 1 + size + 1 + env.length();
  if(!out.write(itoa(fullsize)) || // Start the "outer" netstring
     !out.write(":") ||
     !out.write(itoa(size)) ||	// Start the message netstring
     !out.write(":") ||
     !fdbuf_copy(msg, out, true) || // Send out the message
     !out.write(",") ||		// End the message netstring
     !out.write(env) ||		// The envelope is already encoded
     !out.write(",") ||		// End the "outer" netstring
     !out.flush())
    return -ERR_MSG_WRITE;
  mystring response;
  if(!in.getnetstring(response))
    return -ERR_PROTO;
  switch(response[0]) {
  case 'K': return 0;
  case 'Z': return -ERR_MSG_TEMPFAIL;
  case 'D': return -ERR_MSG_PERMFAIL;
  default: return -ERR_PROTO;
  }
}

bool compute_size(fdibuf& msg, unsigned long& size)
{
  char buf[4096];
  size = 0;
  while(msg.read(buf, 4096))
    size += msg.last_count();
  if(msg.eof())
    size += msg.last_count();
  return size > 0;
}

bool make_envelope(fdibuf& msg, mystring& env)
{
  mystring tmp;
  while(msg.getline(tmp)) {
    if(!tmp)
      return true;
    env += str2net(tmp);
  }
  return false;
}
    
bool preload_data(fdibuf& msg, unsigned long& size, mystring& env)
{
  return make_envelope(msg, env) &&
    compute_size(msg, size);
}
  
int main(int, char* argv[])
{
  fdibuf msg(argv[2]);
  if(!msg)
    return ERR_MSG_OPEN;
  unsigned long size;
  mystring envelope;
  if(!preload_data(msg, size, envelope))
    return ERR_MSG_READ;
  int fd = tcpconnect(argv[1], 628);
  if(fd < 0)
    return -fd;
  alarm(60*60);			// Connection must close after an hour
  qmqp conn(fd);
  return -conn.send(msg, size, envelope);
}
